////////////////////////////////////////////////////////////////////////
/*
     FILE: AVHRR.java
  PURPOSE: Implements the NOAA AVHRR radiometer.
   AUTHOR: Peter Hollemans
     DATE: 2007/02/28
  CHANGES: n/a

  Copyright (C) 2007, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.instrument;

/**
 * The <code>AVHRR</code> is a radiometer for the NOAA Advanced Very
 * High Resolution Radiometer.
 */
public class AVHRR 
  implements Radiometer {

  // Constants
  // ---------

  /** The number of sample value for the AVHRR sensor. */
  public static final int SAMPLES = 2048;

  // Variables
  // ---------

  /** The singleton instance of this class. */
  private static AVHRR instance = new AVHRR();

  ////////////////////////////////////////////////////////////

  private AVHRR () {}

  ////////////////////////////////////////////////////////////

  /** Gets the singleton instance of this class. */
  public static AVHRR getInstance() { return (instance); }

  ////////////////////////////////////////////////////////////

  public String getName() { return ("avhrr"); }

  ////////////////////////////////////////////////////////////

  public String getLongName() { 

    return ("Advanced Very High Resolution Radiometer"); 

  } // getLongName

  ////////////////////////////////////////////////////////////

  public int getChannelCount() { return (6); }

  ////////////////////////////////////////////////////////////

  public String getChannelName (int channel) {
    
    switch (channel) {
    case 1: return ("avhrr_ch1");
    case 2: return ("avhrr_ch2");
    case 3: return ("avhrr_ch3");
    case 4: return ("avhrr_ch4");
    case 5: return ("avhrr_ch5");
    case 6: return ("avhrr_ch3a");
    default: throw new IllegalArgumentException();
    } // switch

  } // getChannelName

  ////////////////////////////////////////////////////////////

  public boolean isThermal (int channel) {

    switch (channel) {
    case 1: case 2: case 6: return (false);
    case 3: case 4: case 5: return (true);
    default: throw new IllegalArgumentException();
    } // switch

  } // isThermal

  ////////////////////////////////////////////////////////////

  public int getSampleCount() { return (SAMPLES); }

  ////////////////////////////////////////////////////////////

} // AVHRR class

////////////////////////////////////////////////////////////////////////
