////////////////////////////////////////////////////////////////////////
/*
     FILE: Instrument.java
  PURPOSE: Interface for data gathering instruments.
   AUTHOR: Peter Hollemans
     DATE: 2007/02/19
  CHANGES: n/a

  Copyright (C) 2007, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.instrument;

// Imports
// -------
import terrenus.telemetry.*;
import terrenus.orbit.*;

/**
 * The <code>Instrument</code> interface is mainly a marker interface for
 * scientific instruments that gather data from a spacecraft.
 *
 * @see TelemetryDecoder#getData
 * @see Spacecraft#getInstruments
 */
public interface Instrument {

  ////////////////////////////////////////////////////////////

  /** Gets the short name of the instrument. */
  public String getName();

  ////////////////////////////////////////////////////////////

  /** Gets the long name of the instrument. */
  public String getLongName();

  ////////////////////////////////////////////////////////////

} // Instrument interface

////////////////////////////////////////////////////////////////////////
