////////////////////////////////////////////////////////////////////////
/*
     FILE: Radiometer.java
  PURPOSE: Interface for multichannel radiometers.
   AUTHOR: Peter Hollemans
     DATE: 2007/02/22
  CHANGES: n/a

  Copyright (C) 2007, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.instrument;

/**
 * The <code>Radiometer</code> interface represents a scientific
 * instrument that gathers image data from multiple channels in
 * the visible and IR ranges.
 */
public interface Radiometer 
  extends Instrument {

  ////////////////////////////////////////////////////////////

  /** Gets the number of radiometer channels. */
  public int getChannelCount();

  ////////////////////////////////////////////////////////////

  /**
   * Gets the radiometer channel name.
   *
   * @param channel the channel in the range [1..N].
   */
  public String getChannelName (int channel);

  ////////////////////////////////////////////////////////////

  /** 
   * Returns true if the channel uses thermal calibration or false
   * for visible. 
   *
   * @param channel the channel in the range [1..N].
   */
  public boolean isThermal (int channel);

  ////////////////////////////////////////////////////////////

  /** Gets the sample count for this radiometer. */
  public int getSampleCount();

  ////////////////////////////////////////////////////////////

} // Radiometer interface

////////////////////////////////////////////////////////////////////////
