////////////////////////////////////////////////////////////////////////
/*
     FILE: HRPTFrameSyncChecker.java
  PURPOSE: Checks NOAA HRPT frame sync.
   AUTHOR: Peter Hollemans
     DATE: 2007/02/20
  CHANGES: n/a

  Copyright (C) 2007, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.telemetry;

// Imports
// -------
import java.nio.*;

/**
 * The <code>HRPTFrameSyncChecker</code> class checks NOAA HRPT
 * frame synchronization data.
 */
public class HRPTFrameSyncChecker 
  extends ShortSignatureChecker {

  // Constants
  // ---------

  /** The 6 expected frame sync words. */
  public static final short[] FRAME_SYNC_WORDS = new short[] {
    (short) 0x284,
    (short) 0x16f,
    (short) 0x35c,
    (short) 0x19d,
    (short) 0x20f,
    (short) 0x95
  };

  // Variables
  // ---------

  /** The static instance of this class. */
  private static HRPTFrameSyncChecker instance = new HRPTFrameSyncChecker();

  ////////////////////////////////////////////////////////////

  /** Gets the single instance of this class. */
  public static HRPTFrameSyncChecker getInstance () { return (instance); }

  ////////////////////////////////////////////////////////////

  /** Creates a new frame sync checker. */
  private HRPTFrameSyncChecker () {

    super (FRAME_SYNC_WORDS, (short) 0x03ff);

  } // HRPTFrameSyncChecker constructor

  ////////////////////////////////////////////////////////////

} // HRPTFrameSyncChecker class

////////////////////////////////////////////////////////////////////////
