////////////////////////////////////////////////////////////////////////
/*
     FILE: SignatureChecker.java
  PURPOSE: Interface for checking data signatures.
   AUTHOR: Peter Hollemans
     DATE: 2007/02/20
  CHANGES: n/a

  Copyright (C) 2007, Terrenus Earth Sciences

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/
////////////////////////////////////////////////////////////////////////

// Package
// -------
package terrenus.telemetry;

// Imports
// -------
import java.nio.*;

/**
 * The <code>SignatureChecker</code> interface is for classes
 * that implement a comparison between a series of data values
 * and a known signature pattern.
 */
public interface SignatureChecker {

  /**
   * Gets the number of bits different between the data and the
   * known signature.
   * 
   * @param buf the data buffer to check.
   *
   * @return the total number of bits different between data and
   * reference.  If zero, the data matches the signature.
   */ 
  public int getDifferenceCount (
    ByteBuffer buf
  );

} // SignatureChecker class

////////////////////////////////////////////////////////////////////////
